﻿
local Widget = LIBTD('Widget')
if not Widget or not Widget('Panel') or Widget('About') then return end

local strlen = strlen
local About = Widget:NewModule('About')
local test = Widget.Button:CreateFontString(nil, nil, 'GameFontHighlightSmall')

do
    local L = Widget:NewLocale('zhCN')
    if L then
        L['About'] = '关于'
        L['Name:'] = '插件名称：'
        L['Version:'] = '插件版本：'
        L['Author:'] = '插件作者：'
        L['Email:'] = '电子邮箱：'
        L['Url:'] = '插件链接：'
    end
    
    L = Widget:NewLocale('zhTW')
    if L then
        L['About'] = '關於'
        L['Name:'] = '插件名稱：'
        L['Version:'] = '插件版本：'
        L['Author:'] = '插件作者：'
        L['Email:'] = '电子郵箱：'
        L['Url:'] = '插件鏈接：'
    end
end

local L = Widget:GetLocale()

local backdrop = {
    bgFile = [[Interface\ChatFrame\ChatFrameBackground]],
    edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]],
    edgeSize = 14, tileSize = 20, tile = true,
    insets = {left = 2, right = 2, top = 2, bottom = 2}
}

local function OnTextChanged(self)
    if self:GetText() ~= self.text then
        self:SetText(self.text)
    end
    if self:HasFocus() then
        self:HighlightText(0, strlen(self.text))
    end
end

local function OnEditFocusGained(self)
    self:HighlightText(0, strlen(self.text))
end

local function OnEditFocusLost(self)
    self:HighlightText(0, 0)
    if not self:IsMouseOver() then
        self:SetBackdrop({})
    end
end

local function OnEnter(self)
    self:SetBackdrop(backdrop)
	self:SetBackdropColor(0, 0, 0, 0.4)
end

local function OnLeave(self)
    if not self:HasFocus() then
        self:SetBackdrop({})
    end
end

local function CreateEditbox(parent, text)
    local editbox = CreateFrame('Editbox', nil, parent)
    
    editbox.text = text
    
    editbox:SetFontObject('GameFontHighlightSmall')
    editbox:SetTextInsets(8, 8, 0, 0)
    editbox:SetAutoFocus(false)
    editbox:SetSize(250, 20)
    editbox:SetScript('OnTextChanged', OnTextChanged)
    editbox:SetScript('OnEditFocusGained', OnEditFocusGained)
    editbox:SetScript('OnEditFocusLost', OnEditFocusLost)
    editbox:SetScript('OnEscapePressed', editbox.ClearFocus)
    editbox:SetScript('OnEnter', OnEnter)
    editbox:SetScript('OnLeave', OnLeave)
    
    editbox:HighlightText(0, 0)
    
    test:SetText(text)
    editbox:SetWidth(test:GetStringWidth() + 32)
    
    return editbox
end

function About:New(parent, addon, email)
    if not addon then return end
    
    local about = Widget('Panel'):New(parent)
    
    about:SetLabelText(L['About'])
    about:CreateLabel('TOPLEFT', 20, -30):SetText(L['Name:'])
    about:CreateLabel('TOPLEFT', 120, -30, nil, true):SetText(addon)
    about:CreateLabel('TOPLEFT', 20, -60):SetText(L['Version:'])
    about:CreateLabel('TOPLEFT', 120, -60, nil, true):SetText(LIBTD(addon).version)
    about:CreateLabel('TOPLEFT', 20, -90):SetText(L['Author:'])
    about:CreateLabel('TOPLEFT', 120, -90, nil, true):SetText('ldz5')
    about:CreateLabel('TOPLEFT', 20, -120):SetText(L['Email:'])
    about:CreateLabel('TOPLEFT', 20, -150):SetText(L['Url:'])
    
    CreateEditbox(about, email or 'ldz5@qq.com'):SetPoint('TOPLEFT', 112, -120)
    CreateEditbox(about, url or 'http://tdaddon.blog.163.com/'):SetPoint('TOPLEFT', 112, -150)
    
    return about
end
