
local Widget = LIBTD('Widget')
if not Widget or Widget('ColorPicker') then return end

local CreateFrame = CreateFrame
local ShowUIPanel = ShowUIPanel
local ColorPickerFrame = ColorPickerFrame

local ColorPicker = Widget:NewModule('ColorPicker', CreateFrame('Button'))
Widget:CopyFunctions(ColorPicker)

function ColorPicker:New(parent, db, ...)
    local button = self:Bind(CreateFrame('Button', nil, parent))
    
	button:SetNormalTexture([[Interface\ChatFrame\ChatFrameColorSwatch]])
	button:SetHighlightTexture([[Interface\Buttons\ButtonHilight-Square]])
    button:SetSize(20, 20)
    button:SetFontString(button:CreateLabel('LEFT', 'RIGHT', 2, 0))

    button:SetDBAndKeys(db, ...)
    
    button:SetScript('OnClick', self.OnClick)
	button:RegisterForClicks('LeftButtonUp')
    
    return button
end
ColorPicker.SetLabelText = ColorPicker.SetText

local function saveColor()
    local r, g, b = ColorPickerFrame:GetColorRGB()
    local button = ColorPickerFrame.button
    
    button.prevColor.r = r
    button.prevColor.g = g
    button.prevColor.b = b
    button:SaveDBValue(button.prevColor)
    
	ColorPickerFrame.button:SetColor(r, g, b)
end

local function undoColor()
    local button = ColorPickerFrame.button
    local color = button.prevColor
    
    button:SaveDBValue(color)
    
    ColorPickerFrame.button:SetColor(color.r, color.g, color.b)
end

function ColorPicker:OnClick()
    if ColorPickerFrame:IsShown() then
        ColorPickerFrame:Hide()
    else
        local color = self:GetDBValue()
        self.prevColor = color
        
        ColorPickerFrame.button = self
        ColorPickerFrame.hasOpacity = nil
        ColorPickerFrame.func = saveColor
        ColorPickerFrame.cancelFunc = undoColor
        ColorPickerFrame:SetColorRGB(color.r, color.g, color.b)
        
        ShowUIPanel(ColorPickerFrame)
    end
end

function ColorPicker:Update()
    local color = self:GetDBValue()
    
    self:SetColor(color.r, color.g, color.b)
end

function ColorPicker:SetColor(r, g, b)
    self:GetNormalTexture():SetVertexColor(r, g, b)
end

function ColorPicker:GetPositionArgs()
    return 40, 0, 0 -- height, y, lx, ly
end

function ColorPicker:GetWidgetType()
    return 'ColorPicker'
end
