
local Widget = LIBTD('Widget')

if not Widget or not Widget('Frame') or Widget('ConfigFrame') then return end

local Frame = Widget('Frame')

local ConfigFrame = Widget:NewModule('ConfigFrame', Frame:New())

local f = CreateFrame('Frame', 'LIBTDMainFrame', UIParent)
tinsert(UISpecialFrames, f:GetName())
f.frames = {}
f:SetScript('OnHide', function() for i, frame in ipairs(f.frames) do frame:Hide() end end)

local backdrop = {
    bgFile = [[Interface\DialogFrame\UI-DialogBox-Background]],
    edgeFile = [[Interface\DialogFrame\UI-DialogBox-Border]],
    tile = true,
    tileSize = 32,
    edgeSize = 32,
    insets = { left = 11, right = 12, top = 12, bottom = 11 }
}

local function CloseClick(self)
    self:GetParent():Hide()
end

function ConfigFrame:New(parent)
    local config = self:Bind(Frame:New(parent))
    
    config:SetBackdrop(backdrop)
    config:SetFrameStrata('DIALOG')
    config:SetToplevel(true)
    config:Hide()

    local titletex = config:CreateTexture(nil, 'ARTWORK')
    titletex:SetTexture([[Interface\DialogFrame\UI-DialogBox-Header]])
    titletex:SetSize(320, 64)
    titletex:SetPoint('TOP', 0, 12)
    
    config.titletex = titletex
    config.title = config:CreateLabel('TOP', 'TOP')
    
    config:SetResizable(true)
    config:SetMovable(true)
    config:EnableMouse(true)
    config:RegisterForDrag('LeftButton')
    config:SetScript('OnShow', self.OnShow)
    config:SetScript('OnDragStart', self.StartMoving)
    config:SetScript('OnDragStop', self.StopMovingOrSizing)
    
    return config
end

function ConfigFrame:EnableEscape()
    tinsert(f.frames, self)
    self.enableEscape = true
    self:SetScript('OnHide', self.OnHide)
end

local origSetPoint = ConfigFrame.SetPoint
function ConfigFrame:SetPoint(...)
    self.points = {...}
    origSetPoint(self, ...)
end

function ConfigFrame:OnShow()
    if self.points then
        self:ClearAllPoints()
        self:SetPoint(unpack(self.points))
    end
    self:Update()
    if self.enableEscape then
        f:Show()
    end
end

function ConfigFrame:OnHide()
    for i, frame in ipairs(f.frames) do
        if frame:IsVisible() then
            return
        end
    end
    f:Hide()
end

function ConfigFrame:EnableClose()
    if not self.close then
        local close = CreateFrame('Button', nil, self, 'UIPanelCloseButton')
        close:SetPoint('TOPRIGHT', -3, -3)
        close:SetScript('OnClick', CloseClick)
        self.close = close
    end
end

function ConfigFrame:SetTitleText(text)
    self.title:SetText(text)
    self.titletex:SetWidth(self.title:GetStringWidth() + 200)
end
ConfigFrame.SetLabelText = ConfigFrame.SetTitleText

function ConfigFrame:GetWidgetType()
    return 'ConfigFrame'
end
