
local Widget = LIBTD('Widget')
if not Widget or Widget('Dialog') then return end

local ceil = ceil
local StaticPopupDialogs = StaticPopupDialogs

local Dialog = Widget:NewModule('Dialog')

local function Input_CallBack(self)
    local dialog = self.editBox and self or self:GetParent()
    if dialog.DialogOnAccept then
        dialog.DialogOnAccept(dialog.editBox:GetText(), dialog.data, dialog.data2)
    end
    dialog:Hide()
end

StaticPopupDialogs['TDLIB_COMFIRM'] = {
	button1 = ACCEPT,
	button2 = CANCEL,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1,
	multiple = 1,
    showAlert = true,
}

StaticPopupDialogs['TDLIB_INPUT'] = {
	button1 = ACCEPT,
	button2 = CANCEL,
	hasEditBox = 1,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1,
	exclusive = 1,
	multiple = 1,
    OnAccept = Input_CallBack,
    EditBoxOnEnterPressed = Input_CallBack,
}

StaticPopupDialogs['TDLIB_WARNING'] = {
	button1 = YES,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1,
	multiple = 1,
    showAlert = true,
}

local CopyText = Widget:NewModule('CopyText', Widget('ConfigFrame'):New(UIParent))

do
    CopyText:SetSize(350, 180)
    CopyText:ClearAllPoints()
    CopyText:SetPoint('CENTER')
    CopyText:SetTitleText('Copy Text')
    CopyText:SetFrameStrata('TOOLTIP')
    
    local editbox = Widget('ScrollEditbox'):New(CopyText)
    editbox:SetPoint('TOPLEFT', 20, -30)
    editbox:SetSize(310, 120)
    editbox.editbox:SetScript('OnEditFocusGained', function(self) self:HighlightText(0, strlen(self:GetText())) end)
    editbox.editbox:SetScript('OnEscapePressed', function() CopyText:Hide() end)
    
    local accept = Widget('Button'):New(CopyText)
    accept:SetPoint('BOTTOMRIGHT', CopyText, 'BOTTOM', -5, 10)
    accept:SetText(ACCEPT)
    accept:SetScript('OnClick', function()
        if CopyText.DialogOnAccept then
            CopyText.DialogOnAccept(editbox:GetText(), CopyText.data, CopyText.data2)
        end
        CopyText:Hide()
    end)
    
    local cancel = Widget('Button'):New(CopyText)
    cancel:SetPoint('BOTTOMLEFT', CopyText, 'BOTTOM', 5, 10)
    cancel:SetText(CANCEL)
    cancel:SetScript('OnClick', function()
        CopyText:Hide()
    end)
    
    CopyText.editbox = editbox
end

function Dialog:CopyText(title, defaulttext, okfunc, data, data2)
    CopyText:SetLabelText(title)
    CopyText.data = data
    CopyText.data2 = data2
    CopyText.DialogOnAccept = okfunc
    CopyText:Show()
    CopyText.editbox:SetText(defaulttext)
    CopyText.editbox:SetFocus()
end

function Dialog:Input(text, maxletters, okfunc, data, data2)
    local dialog = StaticPopupDialogs['TDLIB_INPUT']
    dialog.text = text or 'Enter a string'
    dialog.maxLetters = maxletters
    
    dialog = StaticPopup_Show('TDLIB_INPUT')
    if dialog then
        dialog.data = data
        dialog.data2 = data2
        dialog.DialogOnAccept = okfunc
    end
    return dialog
end

function Dialog:Warning(text, timeout)
    StaticPopupDialogs['TDLIB_WARNING'].timeout = timeout
    StaticPopupDialogs['TDLIB_WARNING'].text = text or 'Warning!'
    
    return StaticPopup_Show('TDLIB_WARNING')
end

function Dialog:Comfirm(text, okfunc, data, data2)
    local dialog = StaticPopupDialogs['TDLIB_COMFIRM']
    dialog.text = text or 'You sure?'
    dialog.OnAccept = okfunc
    
    dialog = StaticPopup_Show('TDLIB_COMFIRM')
    if dialog then
        dialog.data = data
        dialog.data2 = data2
    end
    return dialog
end
