
local Widget = LIBTD('Widget')
if not Widget or Widget('DragFrame') then return end

local CreateFrame = CreateFrame

local DragFrame = Widget:NewModule('DragFrame', CreateFrame('Button'))
local ResizeButton = Widget:NewModule('ResizeButton', CreateFrame('Button'))
Widget:CopyFunctions(DragFrame)
Widget:CopyFunctions(ResizeButton)

function ResizeButton:New(parent)
    local button = self:Bind(CreateFrame('Button', nil, parent))
    
    button:SetSize(16, 16)
    button:SetPoint('BOTTOMRIGHT')
    button:SetNormalTexture([[Interface\ChatFrame\UI-ChatIM-SizeGrabber-Up]])
    button:SetHighlightTexture([[nterface\ChatFrame\UI-ChatIM-SizeGrabber-Highlight]])
	button:SetPushedTexture([[Interface\ChatFrame\UI-ChatIM-SizeGrabber-Down]])
    button:RegisterForDrag('LeftButton')
    
    button:SetScript('OnDragStart', self.OnDragStart)
    button:SetScript('OnDragStop', self.OnDragStop)
    
    return button
end

function ResizeButton:OnDragStart()
    self:SetButtonState('PUSHED', true)
    self:GetParent().parent:StartSizing('BOTTOMRIGHT')
end

function ResizeButton:OnDragStop()
    self:SetButtonState('NORMAL', false)
    self:GetParent().parent:StopMovingOrSizing()
    self:GetParent():SaveSetting()
end

local function LockDrag(self)
    if self.dragFrame then
        self.dragFrame:Hide()
    end
    if self.resizeButton then
        self.resizeButton:Hide()
    end
end

local function UnlockDrag(self)
    if self:IsMovable() and self.dragFrame then
        self.dragFrame:Show()
    end
    
    if self:IsResizable() and self.resizeButton then
        self.resizeButton:Show()
    end
end

local function ToggleDrag(self)
    if self.dragFrame:IsShown() then
        LockDrag(self)
    else
        UnlockDrag(self)
    end
end

function DragFrame:OnDragStart()
    self.parent:ClearAllPoints()
    self.parent:StartMoving()
end

function DragFrame:OnDragStop()
    self.parent:StopMovingOrSizing()
    self:SaveSetting()
end

function DragFrame:SaveSetting()
    local parent = self.parent
    local set = self:GetDBValue()
    if parent:IsMovable() then
        local point, _, rpoint, x, y = parent:GetPoint()
        set.point = point
        set.rpoint = rpoint
        set.x = x
        set.y = y
    end
    if parent:IsResizable() then
        local width, height = parent:GetSize()
        set.width = width
        set.height = height
    end
    
    self:SaveDBValue(set)
end

function DragFrame:New(parent, db, ...)
    local drag = self:Bind(CreateFrame('Button', nil, UIParent))
    
    drag:SetHighlightFontObject('GameFontHighlightSmall')
	drag:SetNormalFontObject('GameFontNormalSmall')
    drag:SetAllPoints(parent)
    drag:Backdrop()
    drag:Hide()
    
    drag:SetScript('OnDragStart', self.OnDragStart)
    drag:SetScript('OnDragStop', self.OnDragStop)
    drag:EnableMouse(true)
    drag:RegisterForDrag('LeftButton')
    drag:SetFrameStrata('TOOLTIP')
    drag:SetClampedToScreen(true)
    
    drag:SetDBAndKeys(db, ...)
    drag.parent = parent
    
    parent:SetClampedToScreen(true)
    parent:SetMovable(true)
    parent:SetResizable(true)
    
    parent.ToggleDrag = ToggleDrag
    parent.dragFrame = drag
    
    parent.resizeButton = ResizeButton:New(drag)
    parent.resizeButton:SetFrameLevel(drag:GetFrameLevel() + 10)
    parent.resizeButton:Hide()
    
    return drag
end
