
local Widget = LIBTD('Widget')
if not Widget or Widget('Dropdown') then return end

local ipairs = ipairs
local CreateFrame = CreateFrame
local ToggleDropDownMenu = ToggleDropDownMenu

local Dropdown = Widget:NewModule('Dropdown', CreateFrame('Button'))
Widget:CopyFunctions(Dropdown)

local DROPDOWN = CreateFrame('Frame', 'LIBTDDropdown', UIParent, 'UIDropDownMenuTemplate')
DROPDOWN.point = 'TOPRIGHT'
DROPDOWN.relativePoint = 'BOTTOMRIGHT'

local currDropdown

local function dropdownClick(self, value, text)
    currDropdown:SaveDBValue(value)
    currDropdown:SetText(text or value)
end

local function dropdownInit()
    if not currDropdown then return end
    
    local info = {}
    for _, v in ipairs(currDropdown.list) do
        info.value = v.value
        info.text  = v.text or v.value
        info.arg1  = v.value
        info.arg2  = v.text
        info.func  = dropdownClick
        info.owner = currDropdown
        info.checked = v.value == currDropdown:GetDBValue() and true or nil
        UIDropDownMenu_AddButton(info, 1)
    end
end

local function buttonOnClick(self)
    currDropdown = self:GetParent()
    ToggleDropDownMenu(1, nil, DROPDOWN, self, 0, 0)
end

UIDropDownMenu_Initialize(DROPDOWN, dropdownInit, 'MENU')

function Dropdown:New(parent, db, ...)
    local dropdown = self:Bind(CreateFrame('Button', nil, parent))
    
    dropdown:Backdrop()
    dropdown:SetHeight(20)
    dropdown:SetNormalFontObject('GameFontHighlightSmall')
    dropdown:SetHighlightTexture([[Interface\TokenFrame\UI-TokenFrame-CategoryButton]])
    dropdown:GetHighlightTexture():SetTexCoord(0, 1, 0.609375, 0.796875)
    dropdown:GetHighlightTexture():SetAlpha(0.5)
    
    local button = CreateFrame('Button', nil, dropdown)
    button:SetSize(22, 22)
    button:SetPoint('LEFT', dropdown, 'RIGHT', -3, 0)
    button:SetNormalTexture([[Interface\ChatFrame\UI-ChatIcon-ScrollDown-Up]])
    button:SetPushedTexture([[Interface\ChatFrame\UI-ChatIcon-ScrollDown-Down]])
    button:SetHighlightTexture([[Interface\Buttons\UI-Common-MouseHilight]])
    button:RegisterForClicks('LeftButtonUp', 'RightButtonUp')
    
    button:SetScript('OnClick', buttonOnClick)
    
    dropdown.tdLabel = dropdown:CreateLabel('BOTTOMLEFT', 'TOPLEFT', 0, 0)
    
    dropdown:SetScript('OnClick', self.OnClick)
    
    dropdown:SetDBAndKeys(db, ...)
    
    return dropdown
end

function Dropdown:OnClick()
    currDropdown = self
    ToggleDropDownMenu(1, nil, DROPDOWN, self, 0, 0)
end

function Dropdown:SetLabelText(text)
    self.tdLabel:SetText(text)
end

function Dropdown:SetList(list)
    self.list = list
end

function Dropdown:GetShowText(value)
    for i, v in ipairs(self.list) do
        if value == v.value then
            return v.text or value
        end
    end
    return value
end

function Dropdown:Update()
    local value = self:GetDBValue()
    self:SetText(self:GetShowText(value))
end

function Dropdown:GetPositionArgs()
    return 40, -15, 0, -20 -- height, y, lx, rx
end

function Dropdown:GetWidgetType()
    return 'Dropdown'
end
