

local Widget = LIBTD('Widget')
if not Widget or not Widget('Panel') or Widget('Listbox') then return end

local CreateFrame = CreateFrame
local type, tinsert, tremove, floor = type, tinsert, tremove, floor

local Panel = Widget('Panel')
local Dialog = Widget('Dialog')
local ScrollBar = Widget('ScrollBar')
local L = Widget:GetLocale()

local Listbox    = Widget:NewModule('Listbox', Panel:New()) Listbox:Hide()
local ItemButton = Widget:NewModule('ListboxItemButton', CreateFrame('CheckButton'))

local UpButton = Widget:NewModule('ListboxItemButtonUpButton', CreateFrame('Button'))
local DownButton = Widget:NewModule('ListboxItemButtonDownButton', CreateFrame('Button'))
local RemoveButton = Widget:NewModule('ListboxItemButtonRemoveButton', CreateFrame('Button'))
Widget:CopyFunctions(UpButton, 'SetNote')
Widget:CopyFunctions(DownButton, 'SetNote')
Widget:CopyFunctions(RemoveButton, 'SetNote')
Widget:CopyFunctions(ItemButton, 'CreateLabel', 'SetNote')

Widget:CopyFunctions(Listbox)

local function OnEnter(self)
    self:GetParent():LockHighlight()
end

local function OnLeave(self)
    self:GetParent():UnlockHighlight()
end
------------------------------------------------------------------------------- up button

function UpButton:New(parent)
    local button = self:Bind(CreateFrame('Button', nil, parent))
    
	button:SetSize(16, 16)
	button:SetNormalTexture([[Interface\Buttons\UI-ScrollBar-ScrollUpButton-Up]])
	button:SetPushedTexture([[Interface\Buttons\UI-ScrollBar-ScrollUpButton-Down]])
	button:SetDisabledTexture([[Interface\Buttons\UI-ScrollBar-ScrollUpButton-Disabled]])
	button:SetHighlightTexture([[Interface\Buttons\UI-ScrollBar-ScrollUpButton-Highlight]])
    
	button:GetNormalTexture():SetTexCoord(1/4, 3/4, 1/4, 3/4)
	button:GetPushedTexture():SetTexCoord(1/4, 3/4, 1/4, 3/4)
	button:GetDisabledTexture():SetTexCoord(1/4, 3/4, 1/4, 3/4)
	button:GetHighlightTexture():SetTexCoord(1/4, 3/4, 1/4, 3/4)
	button:GetHighlightTexture():SetBlendMode('ADD')
    
    button:SetScript('OnClick', self.OnClick)
    button:SetNote(parent:GetParent().upNote)
    
    return button
end

function UpButton:OnClick()
    local button = self:GetParent()
    local listbox = button:GetParent()
    
    local index = button:GetIndex()
    local list = listbox:GetList()
    
    list[index], list[index - 1] = list[index - 1], list[index]
    
    if Widget.callback[listbox.db] then
        Widget.callback[listbox.db]()
    end
    
    listbox:Update()
end

UpButton.OnEnter = OnEnter
UpButton.OnLeave = OnLeave

------------------------------------------------------------------------------- down button

function DownButton:New(parent)
    local button = self:Bind(CreateFrame('Button', nil, parent))
    
	button:SetPoint('RIGHT', upButton, 'LEFT', -2, 0)
	button:SetSize(16, 16)
	button:SetNormalTexture([[Interface\Buttons\UI-ScrollBar-ScrollDownButton-Up]])
	button:SetPushedTexture([[Interface\Buttons\UI-ScrollBar-ScrollDownButton-Down]])
	button:SetDisabledTexture([[Interface\Buttons\UI-ScrollBar-ScrollDownButton-Disabled]])
	button:SetHighlightTexture([[Interface\Buttons\UI-ScrollBar-ScrollDownButton-Highlight]])

	button:GetNormalTexture():SetTexCoord(1/4, 3/4, 1/4, 3/4)
	button:GetPushedTexture():SetTexCoord(1/4, 3/4, 1/4, 3/4)
	button:GetDisabledTexture():SetTexCoord(1/4, 3/4, 1/4, 3/4)
	button:GetHighlightTexture():SetTexCoord(1/4, 3/4, 1/4, 3/4)
	button:GetHighlightTexture():SetBlendMode('ADD')
    
    button:SetScript('OnClick', self.OnClick)
    button:SetNote(parent:GetParent().downNote)
    
    return button
end

function DownButton:OnClick()
    local button = self:GetParent()
    local listbox = button:GetParent()
    
    local index = button:GetIndex()
    local list = listbox:GetList()
    
    list[index], list[index + 1] = list[index + 1], list[index]
    
    if Widget.callback[listbox.db] then
        Widget.callback[listbox.db]()
    end
    listbox:Update()
end

DownButton.OnEnter = OnEnter
DownButton.OnLeave = OnLeave

------------------------------------------------------------------------------- remove button

function RemoveButton:New(parent)
    local button = self:Bind(CreateFrame('Button', nil, parent, 'UIPanelCloseButton'))
    
	button:SetSize(26, 26)
	button:SetScript('OnClick', self.OnClick)
    button:SetNote(parent:GetParent().removeNote)
    
    return button
end

local function removeItem(self, button, item)
    if not button then return end
    
    item = item or button:GetItem()
    if not item then return end
    
    local listbox = button:GetParent()
    
    listbox.selected = nil
    
    local list = listbox:GetList()
    for i, t in ipairs(list) do
        if listbox:GetShowText(t) == listbox:GetShowText(item) then
            tremove(list, i)
            break
        end
    end
    
    if listbox.RemoveClick then
        listbox.RemoveClick(button, item)
    end
    if Widget.callback[listbox.db] then
        Widget.callback[listbox.db]()
    end
    listbox:Update()
end

function RemoveButton:OnClick()
    local button = self:GetParent()
    Dialog:Comfirm(self.note, removeItem, button, button:GetItem())
end

RemoveButton.OnEnter = OnEnter
RemoveButton.OnLeave = OnLeave

------------------------------------------------------------------------------- list button

function ItemButton:New(parent)
    local button = self:Bind(CreateFrame('CheckButton', nil, parent))
    
	button:SetHeight(parent and parent:GetItemHeight() or 25)
    button:Hide()
    
    button:SetHighlightTexture([[Interface\QuestFrame\UI-QuestTitleHighlight]])
	button:SetCheckedTexture([[Interface\QuestFrame\UI-QuestTitleHighlight]])
    
    button:SetScript('OnClick', self.OnClick)
    button:SetFontString(button:CreateLabel('LEFT'))
    button:GetFontString():SetPoint('RIGHT')
    
    button.removeButton = RemoveButton:New(button)
    button.upButton = UpButton:New(button)
    button.downButton = DownButton:New(button)
    button.removeButton:SetPoint('RIGHT', 0, 0)
    button.downButton:SetPoint('RIGHT', button.removeButton, 'LEFT', 5, 0)
    button.upButton:SetPoint('RIGHT', button.downButton, 'LEFT', -2, 0)
    
    return button
end

function ItemButton:OnClick()
    local parent = self:GetParent()
    if self.click then
        if parent.SetChecked then
            parent:SetChecked(parent:GetSelected())
        end
        return self:click()
    elseif parent.SetSelected then
        parent:SetSelected(self:GetIndex())
    end
end

function ItemButton:UpdateExtraButton(up, down, hasRemove)
    if hasRemove then
        self.removeButton:Show()
    else
        self.removeButton:Hide()
    end
    if up then
        self.upButton:Show()
    else
        self.upButton:Hide()
    end
    if down then
        self.downButton:Show()
    else
        self.downButton:Hide()
    end
end

function ItemButton:SetItem(item)
    self.item = item
end

function ItemButton:GetItem(item)
    return self.item
end

function ItemButton:SetIndex(index)
    self.index = index
end

function ItemButton:GetIndex()
    return self.index
end

function ItemButton:SetClick(func)
    self.click = func
end

------------------------------------------------------------------------------- list box

function Listbox:New(parent, db, ...)
    local listbox = self:Bind(Panel:New(parent))
    if not parent then return listbox end
    
    local scrollbar = ScrollBar:New(listbox)
    listbox.scrollbar = scrollbar
    listbox.itembuttons = {}
    
    listbox:EnableMouseWheel(true)
    listbox:SetScript('OnMouseWheel', self.OnMouseWheel)
    listbox:SetScript('OnSizeChanged', self.OnSizeChanged)
    
    listbox:SetDBAndKeys(db, ...)
    
    return listbox
end

function Listbox:SetList(list)
    self.list = list
end

function Listbox:SetExtraList(list)
    self.extralist = list
end

function Listbox:GetList()
    return self.list
end

function Listbox:GetExtraList()
    return self.extralist
end

function Listbox:GetListCount()
    return self.list and #self.list or 0
end

function Listbox:GetExtraListCount()
    return self.extralist and #self.extralist or 0
end

function Listbox:GetItem(index)
    if not index or not self.list then return end
    
    local listCount = self:GetListCount()
    if index <= listCount then
        return self.list[index]
    else
        return self.extralist[index - listCount]
    end
end

function Listbox:GetItemCount()
    return self:GetListCount() + self:GetExtraListCount()
end

function Listbox:GetExtraTag(index)
    if not index then return end
    
    local listCount = self:GetListCount()
    if index > listCount then
        return
    else
        return self.hasUpdown and index ~= 1, self.hasUpdown and index ~= listCount, self.hasRemove
    end
end

function Listbox:SetStartIndex(index)
    self.startIndex = index
end

function Listbox:GetStartIndex()
    local count = self:GetItemCount()
    
    if not self.maxCount then
        self:OnSizeChanged(self:GetSize())
    end
    
    if self.startIndex and count > (self.maxCount or 0) and self.startIndex + self.maxCount > count then
        self.startIndex = count - self.maxCount + 1
    end
    return self.startIndex or 1
end

function Listbox:GetEndIndex()
    return min(self:GetStartIndex() + (self.maxCount or 999) - 1, self:GetItemCount())
end

function Listbox:GetShowText(item)
    if not item then
        return ''
    end
    
    local t = type(item)
    if t == 'table' then
        return item.text or item.name or item.value
    else
        return item
    end
end

function Listbox:UpdateScrollBar(value, minvalue, maxvalue)
    if not self.scrollbar then return end
    
    local min, max = self.scrollbar:GetMinMaxValues()
    if min ~= minvalue or max ~= maxvalue then
        self.scrollbar:SetMinMaxValues(minvalue, maxvalue)
    end
    
    if value ~= self.scrollbar:GetValue() then
        self.scrollbar:SetValue(value)
    end
    
    if self.itembuttons[1] then
        if minvalue == maxvalue then
            self.itembuttons[1]:SetPoint('TOPRIGHT', 0, -10)
        else
            self.itembuttons[1]:SetPoint('TOPRIGHT', -26, -10)
        end
    end
end

function Listbox:Update()
    local button, item
    
    local count = 1
    local s, e = self:GetStartIndex(), self:GetEndIndex()
    
    for index = s, e do
        item = self:GetItem(index)
        
        button = self:GetButton(count)
        button:SetIndex(index)
        button:UpdateExtraButton(self:GetExtraTag(index))
        button:SetClick(type(item) == 'table' and item.click)
        button:SetText(self:GetShowText(item))
        button:SetItem(item)
        button:Show()
        
        if self.UpdateButton then
            self:UpdateButton(button, item)
        end
        
        button.item = item
        count = count + 1
    end
    
    local maxvalue = self.maxCount and self:GetItemCount() - self.maxCount + 1
    maxvalue = (maxvalue and maxvalue >= 1) and maxvalue or 1
    
    self:UpdateScrollBar(s, 1, maxvalue)
    
    if count <= #self.itembuttons then
        for i = count, #self.itembuttons do
            self.itembuttons[i]:Hide()
        end
    end
    
    self:SetChecked(self:GetSelected())
end

function Listbox:GetButton(i)
    return self.itembuttons[i] or self:NewButton(button)
end

function Listbox:NewButton()
    local button = ItemButton:New(self)
    tinsert(self.itembuttons, button)
    
    local index = #self.itembuttons
    if index == 1 then
        button:SetPoint('TOPLEFT', 10, -10)
        button:SetPoint('TOPRIGHT', 0, -10)
    else
        button:SetPoint('TOPLEFT', self.itembuttons[index - 1], 'BOTTOMLEFT', 0, 0)
        button:SetPoint('TOPRIGHT', self.itembuttons[index - 1], 'BOTTOMRIGHT', 0, 0)
    end
    
    if self.ButtonEnter then
        button:SetNote(1)
        button:SetScript('OnEnter', self.ButtonEnter)
    end
    
    if self.FormatButton then
        self:FormatButton(button)
    end
    
    return button
end

function Listbox:SetChecked(index)
    for i, button in ipairs(self.itembuttons) do
        button:SetChecked(false)
    end
    if self.enableSelect then
        self.selected = index
        for i, button in ipairs(self.itembuttons) do
            if index == button:GetIndex() then
                return button:SetChecked(true)
            end
        end
    end
end

function Listbox:SetSelected(index)
    self:SetChecked(index)
    if self.enableSelect and self.OnSelectChanged then
        self:OnSelectChanged(index)
    end
end

function Listbox:SetSelectedByValue(text)
    for i = 1, self:GetItemCount() do
        if text == self:GetItemValue(self:GetItem(i)) then
            return self:SetSelected(i)
        end
    end
end

function Listbox:GetSelected()
    return self.selected
end

function Listbox:GetItemValue(item)
    if type(item) == 'table' then
        return item.value or item.text or item.name
    else
        return item
    end
end

function Listbox:GetSelectedValue()
    return self:GetItemValue(self:GetItem(self.selected))
end

function Listbox:SetExtraButton(hasUpdown, hasRemove)
    self.hasUpdown = hasUpdown
    self.hasRemove = hasRemove
end

function Listbox:SetExtraNote(upNote, downNote, removeNote)
    self.upNote, self.downNote, self.removeNote = upNote, downNote, removeNote
end

function Listbox:EnableSelect(enable)
    self.enableSelect = enable
end

function Listbox:GetWidgetType()
    return 'Listbox'
end

function Listbox:OnScrollValueChanged(value)
    self:SetStartIndex(value)
    self:Update()
end

function Listbox:OnSizeChanged(width, height)
    self.maxCount = floor((height - 10) / self:GetItemHeight())
    self:SetWheelStep(floor(self.maxCount / 2))
    self:Update()
end

function Listbox:SetItemHeight(height)
    self.itemHeight = height
end

function Listbox:GetItemHeight()
    return self.itemHeight or 25
end
