

local Widget = LIBTD('Widget')
if not Widget or not Widget('Frame') or Widget('Panel') then return end

local ceil = ceil
local CreateFrame = CreateFrame

local Frame = Widget('Frame')
local ScrollBar = Widget('ScrollBar')
local TabButton = Widget('TabButton')
local Panel = Widget:NewModule('Panel', Frame:New()) Panel:Hide()
Widget:CopyFunctions(Frame, 'Into', 'IsWidgetType', 'Backdrop')

local TOP, LEFT, RIGHT = 10, 10, -10

function Panel:New(parent)
    local panel = self:Bind(Frame:New(parent))
    if not parent then return panel end
    
    if parent and parent.IsWidgetType and parent:IsWidgetType('TabGroup') then
        panel.button = TabButton:New(parent)
        panel.button.panel = panel
        panel.tdLabel = panel.button
    else
        panel.tdLabel = panel:CreateLabel('BOTTOMLEFT', 'TOPLEFT', 0, 0)
    end
    
	panel:Backdrop()
    panel.yOffset = 0
    panel.scrollStep = 40
    panel.wheelStep = 1
    
    return panel
end

function Panel:SetLabelText(text)
    self.tdLabel:SetText(text)
end

function Panel:AddWidget(widget, height, y, lx, rx)
    if not widget then return end
    
    if self.child then
        widget:SetParent(self.child)
    end
    
    if not height and widget.GetPositionArgs then
        height, y, lx, rx = widget:GetPositionArgs()
    end
    
    if height then
        widget:ClearAllPoints()
        if lx then
            widget:SetPoint('TOPLEFT', LEFT + lx, - TOP - self.yOffset + y)
        end
        if rx then
            widget:SetPoint('TOPRIGHT', RIGHT + rx, - TOP - self.yOffset + y)
        end
        self.yOffset = self.yOffset + height
    end
    
    tinsert(self.widgets, widget)
end

function Panel:GetWidgetType()
    return 'Panel'
end

function Panel:EnableScroll(child, scrollStep)
    self:EnableMouseWheel(true)
    self:SetScript('OnMouseWheel', self.OnMouseWheel)
    self:SetScript('OnScrollRangeChanged', self.OnScrollRangeChanged)
    self:SetScript('OnSizeChanged', self.OnScrollRangeChanged)
    
    if not child then
        child = CreateFrame('Frame', nil, self)
        child:SetPoint('CENTER')
        child:SetSize(100, 100)
    end
    
    self:SetScrollChild(child)
    self.child = child
    self.scrollStep = scrollStep or 40
    self.scrollbar = ScrollBar:New(self)
end

function Panel:SetScrollStep(step)
    self.scrollStep = step
end

function Panel:GetScrollStep()
    return self.scrollStep
end

function Panel:SetWheelStep(step)
    self.wheelStep = step
end

function Panel:GetWheelStep()
    return self.wheelStep
end

function Panel:OnMouseWheel(y)
    self.scrollbar:SetValue(self.scrollbar:GetValue() - y * self.wheelStep)
end

function Panel:OnScrollValueChanged(value)
    self:SetVerticalScroll(value * self.scrollStep)
end

function Panel:OnScrollRangeChanged()
    local range = ceil(self:GetVerticalScrollRange() / self.scrollStep)
    
    local value = self.scrollbar:GetValue()
    if value > range then value = range end
    
    self.scrollbar:SetMinMaxValues(0, range)
    self.scrollbar:SetValue(value)
    
    if self.child:IsObjectType('Editbox') then return end
    
    local width, height = self:GetSize()
    if range <= 0 then
        self.scrollbar:Hide()
        self.child:SetSize(width, height)
    else
        self.scrollbar:Show()
        self.child:SetSize(width - 20, height)
    end
end
