
local Widget = LIBTD('Widget')

if not Widget or Widget('ScrollBar') then return end

local ScrollBar = Widget:NewModule('ScrollBar', CreateFrame('Slider'))


local function UpClick(self)
    local parent = self:GetParent()
    parent:SetValue(parent:GetValue() - 1)
end

local function DownClick(self)
    local parent = self:GetParent()
    parent:SetValue(parent:GetValue() + 1)
end

function ScrollBar:New(parent)
	local f = self:Bind(CreateFrame('Slider', nil, parent))
	f:SetWidth(16)

	f:SetPoint('TOPRIGHT', -8, -24)
	f:SetPoint('BOTTOMRIGHT', -8, 24)
    f:SetValueStep(1)

	local up = CreateFrame('Button', nil, f)
	up:SetPoint('BOTTOM', f, 'TOP')
	up:SetSize(16, 16)
	up:SetNormalTexture([[Interface\Buttons\UI-ScrollBar-ScrollUpButton-Up]])
	up:SetPushedTexture([[Interface\Buttons\UI-ScrollBar-ScrollUpButton-Down]])
	up:SetDisabledTexture([[Interface\Buttons\UI-ScrollBar-ScrollUpButton-Disabled]])
	up:SetHighlightTexture([[Interface\Buttons\UI-ScrollBar-ScrollUpButton-Highlight]])

	up:GetNormalTexture():SetTexCoord(1/4, 3/4, 1/4, 3/4)
	up:GetPushedTexture():SetTexCoord(1/4, 3/4, 1/4, 3/4)
	up:GetDisabledTexture():SetTexCoord(1/4, 3/4, 1/4, 3/4)
	up:GetHighlightTexture():SetTexCoord(1/4, 3/4, 1/4, 3/4)
	up:GetHighlightTexture():SetBlendMode('ADD')

	up:SetScript('OnClick', UpClick)

	local down = CreateFrame('Button', nil, f)
	down:SetPoint('TOP', f, 'BOTTOM')
	down:SetSize(16, 16)
	down:SetNormalTexture([[Interface\Buttons\UI-ScrollBar-ScrollDownButton-Up]])
	down:SetPushedTexture([[Interface\Buttons\UI-ScrollBar-ScrollDownButton-Down]])
	down:SetDisabledTexture([[Interface\Buttons\UI-ScrollBar-ScrollDownButton-Disabled]])
	down:SetHighlightTexture([[Interface\Buttons\UI-ScrollBar-ScrollDownButton-Highlight]])

	down:GetNormalTexture():SetTexCoord(1/4, 3/4, 1/4, 3/4)
	down:GetPushedTexture():SetTexCoord(1/4, 3/4, 1/4, 3/4)
	down:GetDisabledTexture():SetTexCoord(1/4, 3/4, 1/4, 3/4)
	down:GetHighlightTexture():SetTexCoord(1/4, 3/4, 1/4, 3/4)
	down:GetHighlightTexture():SetBlendMode('ADD')

	down:SetScript('OnClick', DownClick)

	f:SetThumbTexture([[Interface\Buttons\UI-ScrollBar-Knob]])
	local thumb = f:GetThumbTexture()
	thumb:SetSize(16, 24)
	thumb:SetTexCoord(1/4, 3/4, 1/8, 7/8)

	f:SetScript('OnValueChanged', self.OnValueChanged)

	local border = CreateFrame('Frame', nil, f)
	border:SetPoint('TOPLEFT', up, -5, 5)
	border:SetPoint('BOTTOMRIGHT', down, 5, -3)
	border:SetBackdrop({
        edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]],
        tile = true,
        tileSize = 16,
        edgeSize = 12,
        insets = { left = 0, right = 0, top = 5, bottom = 5 }
    })
	border:SetBackdropBorderColor(TOOLTIP_DEFAULT_COLOR.r, TOOLTIP_DEFAULT_COLOR.g, TOOLTIP_DEFAULT_COLOR.b, 0.5)
    
    f.up = up
    f.down = down
    f.parent = parent
    
	return f
end

local orig_ScrollBar_SetMinMaxValues = ScrollBar.SetMinMaxValues
function ScrollBar:SetMinMaxValues(min, max)
    if min == max then
        self:Hide()
    else
        self:Show()
    end
    orig_ScrollBar_SetMinMaxValues(self, min, max)
    self:UpdateButton(self:GetValue(), min, max)
end

function ScrollBar:UpdateButton(value, min, max)
    if value == min then
        self.up:Disable()
    else
        self.up:Enable()
    end
    if value == max then
        self.down:Disable()
    else
        self.down:Enable()
    end
end

function ScrollBar:OnValueChanged(value)
    self:UpdateButton(value, self:GetMinMaxValues())
    
    if self.parent.OnScrollValueChanged then
        self.parent:OnScrollValueChanged(value)
    end
end
