
local Widget = LIBTD('Widget')

if not Widget or not Widget('Frame') or Widget('TabGroup') then return end

local CreateFrame = CreateFrame

local Frame = Widget('Frame')
local Panel = Widget('Panel')

local TabGroup = Widget:NewModule('TabGroup', Frame:New()) TabGroup:Hide()
local TabButton = Widget:NewModule('TabButton', CreateFrame('Button'))
Widget:CopyFunctions(TabButton, 'Into')

------------------------------------------------------------- tab group

function TabGroup:New(parent)
    local group = self:Bind(Frame:New(parent))
    
    group.buttons = {}
    group:SetScript('OnShow', self.UpdateTabs)
    
    return group
end

function TabGroup:AddWidget(panel)
    if #self.buttons == 0 then
        panel.button:SetPoint('BOTTOMLEFT', self, 'TOPLEFT', 0, -1)
    else
        panel.button:SetPoint('LEFT', self.buttons[#self.buttons], 'RIGHT', -15, 0)
    end
    panel.button:Show()
    
    panel:SetAllPoints(self)
    
    tinsert(self.widgets, panel)
    tinsert(self.buttons, panel.button)
end

function TabGroup:UpdateTabs(tab)
    self.selected = tab or self.selected
    
    if not self.selected or self.selected.isDisabled then
        for i, button in ipairs(self.buttons) do
            if not button.isDisabled then
                self.selected = button
                break
            end
        end
    end
    
    for i, button in ipairs(self.buttons) do
        
        if button == self.selected then
            PanelTemplates_SelectTab(button)
            button.panel:Show()
        else
            button.panel:Hide()
            if button.isDisabled then
                PanelTemplates_SetDisabledTabState(button)
            else
                PanelTemplates_DeselectTab(button)
            end
        end
    end
end

function TabGroup:Update()
    self:UpdateTabs()
    Frame.Update(self)
end

function TabGroup:SetSelectedByText(text)
    for i, button in ipairs(self.buttons) do
        if button:GetText() == text then
            return self:UpdateTabs(button)
        end
    end
end

function TabGroup:SetSelectedByIndex(index)
    self:UpdateTabs(self.buttons[index])
end

function TabGroup:EnableByText(text)
    for i, button in ipairs(self.buttons) do
        if button:GetText() == text then
            button.isDisabled = nil
            return self:UpdateTabs()
        end
    end
end

function TabGroup:DisableByText(text)
    for i, button in ipairs(self.buttons) do
        if button:GetText() == text then
            button.isDisabled = true
            return self:UpdateTabs()
        end
    end
end

function TabGroup:GetWidgetType()
    return 'TabGroup'
end

------------------------------------------------------------- tab button

function TabButton:New(parent)
    self.index = (self.index or 0) + 1
    local button = self:Bind(CreateFrame('Button', 'LibTDTabButton' .. self.index, parent, 'OptionsFrameTabButtonTemplate'))
    
    button:SetScript('OnClick', self.OnClick)
    
    return button
end

function TabButton:OnClick()
    self:GetParent():UpdateTabs(self)
end
