
local VERSION = 9

local Widget = LIBTD('Widget')
if Widget and Widget.version >= VERSION then return end

local Widget = LIBTD:NewAddon('Widget', nil, VERSION)

local _G, type, select, tinsert, pairs, ipairs, wipe = _G, type, select, tinsert, pairs, ipairs, wipe

local backdrop = {
    bgFile = [[Interface\ChatFrame\ChatFrameBackground]],
    edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]],
    edgeSize = 14, tileSize = 20, tile = true,
    insets = {left = 2, right = 2, top = 2, bottom = 2},
}

local function OnEnter(self)
    if self.OnEnter then
        self:OnEnter(self)
    end
    GameTooltip:SetOwner(self, 'ANCHOR_TOPLEFT')
    GameTooltip:SetText(self.note, 0.5, 1, 1)
    GameTooltip:Show()
end

local function OnLeave(self)
    if self.OnLeave then
        self:OnLeave(self)
    end
    GameTooltip:Hide()
end
    
local functions = {
    SetDB = function(self, db)
        if db and type(db) == 'string' and _G[db] then
            self.db = db
            self.dbkeys = {}
        end
    end,
    
    InsertKey = function(self, ...)
        if not self.dbkeys then return end
        local key
        for i = 1, select("#", ...) do
            key = select(i, ...)
            tinsert(self.dbkeys, key)
        end
    end,
    
    SetKeys = function(self, ...)
        self:ClearKeys()
        self:InsertKey(...)
    end,
    
    SetKey = function(self, key, index)
        index = index or 1
        if self.dbkeys[index] or self.dbkeys[index - 1] then
            self.dbkeys[index] = key
        end
    end,
    
    ClearKeys = function(self)
        self.dbkeys = self.dbkeys or {}
        wipe(self.dbkeys)
    end,
    
    SetDBAndKeys = function(self, db, ...)
        self:SetDB(db)
        self:SetKeys(...)
    end,
    
    SetNote = function(self, note)
        if note then
            self.note = note
            self:SetScript('OnEnter', OnEnter)
            self:SetScript('OnLeave', OnLeave)
        end
    end,
    
    GetDBValue = function(self)
        local db = _G[self.db]
        if not db then return end
        
        for i, key in ipairs(self.dbkeys) do
            db = db[key]
        end
        return Widget:CopyTable(db)
    end,
    
    SaveDBValue = function(self, value)
        local db = _G[self.db]
        if not db then return end
        
        for i, key in ipairs(self.dbkeys) do
            if i == #self.dbkeys then
                db[key] = Widget:CopyTable(value)
                break
            else
                db = db[key]
            end
        end
        
        if Widget.callback[self.db] then
            Widget.callback[self.db]()
        end
    end,
    
    CreateLabel = function(self, point, rpoint, x, y, isValue)
        local label = self:CreateFontString(nil, 'OVERLAY', isValue and 'GameFontHighlightSmall' or 'GameFontNormalSmall')
        label:SetJustifyH('LEFT')
        label:SetHeight(18)
        label:SetPoint(point, self, rpoint, x, y)
        
        return label
    end,
    
    Into = function(self, height, y, lx, rx)
        local parent = self:GetParent()
        if parent and parent.AddWidget then 
            parent:AddWidget(self, height, y, lx, rx)
        end
    end,
    
    IsWidgetType = function(self, widgettype)
        return self:GetWidgetType() == widgettype
    end,
    
    Backdrop = function(self)
        self:SetBackdrop(backdrop)
        self:SetBackdropColor(0, 0, 0, .4)
        self:SetBackdropBorderColor(1, 1, 1, 1)
    end
}

Widget.callback = {}

function Widget:CopyFunctions(module, ...)
    if not module and type(module) ~= 'table' then return end
    
    local list = {...}
    if #list > 0 then
        for i, name in pairs(list) do
            module[name] = functions[name]
        end
    else
        for name, func in pairs(functions) do
            module[name] = func
        end
    end
end

function Widget:SetSaveCallBack(db, func)
    if not db or not func or type(func) ~= 'function' then return end
    
    self.callback[db] = func
end

local oldfuncs = {}
local function setScript(self, script, func)
    local wtype = self:GetWidgetType()
    if oldfuncs[wtype].list[script] then
        self['____'..script] = func
        return
    end
    return oldfuncs[wtype].SetScript(self, script, func)
end

local function getScript(self, script)
    if oldfuncs[wtype].list[script] then
        return self['____'..script]
    end
    return oldfuncs[wtype].GetScript(self, script)
end

function Widget:HookWidgetScript(widget, list)
    oldfuncs[widget:GetWidgetType()] = {
        SetScript = widget.SetScript,
        GetScript = widget.GetScript,
        HasScript = widget.HasScript,
        HookScript = widget.HookScript,
        list = list
    }
    widget.SetScript = setScript
    widget.GetScript = getScript
    widget.HasScript = getScript
    widget.HookScript = setScript
end