
local Addon = LIBTD:NewAddon(...)

local Widget = LIBTD('Widget')

local ErrorFrame = Addon:NewModule('ErrorFrame', Widget('ConfigFrame'):New(UIParent), 'Event')
local MinimapButton = Addon:NewModule('MinimapButton', CreateFrame('Button', nil, Minimap), 'Update')

local L = Addon:GetLocale()

local errorlist = {}
local config

function Addon:OnInit()
    TDDB_TDERRORFRAME = TDDB_TDERRORFRAME or {}
    config = TDDB_TDERRORFRAME
    
    self:InitModule('ErrorFrame')
    self:InitModule('MinimapButton')
end

function ErrorFrame:ADDON_ACTION_BLOCKED(addon, port)
    self:AddError(format(L['%s blocked from using %s'], addon, port), 4)
end

function ErrorFrame:ADDON_ACTION_FORBIDDEN(port)
    BaudErrorFrameAdd(format(L['Macro blocked from using %s'], port), 4)
end

function ErrorFrame:MACRO_ACTION_BLOCKED(addon, port)
    BaudErrorFrameAdd(format(L['%s forbidden from using %s (Only usable by Blizzard)'], addon, port), 4)
end

function ErrorFrame:MACRO_ACTION_FORBIDDEN(port)
    BaudErrorFrameAdd(format(L['Macro forbidden from using %s (Only usable by Blizzard)'], port), 4)
end

function ErrorFrame:AddError(err, retrace)
    for i, v in ipairs(errorlist) do
        if v.text == err then
            v.count = v.count + 1
            return
        end
    end
    tinsert(errorlist, {text = err, count = 1, stack = debugstack(retrace)})
    self:Update()
end

local MESSAGE_FORMAT = '%s\n'..L['Count:']..' %d\n\n'..L['Call Stack:']..'\n%s'

function ErrorFrame:GetMessage(index)
    local info = errorlist[index]
    if info then
        return format(MESSAGE_FORMAT, info.text, info.count, info.stack)
    end
    return ''
end

function ErrorFrame:Update()
    if #errorlist > 0 then
        MinimapButton.text:SetText(#errorlist)
        MinimapButton:Show()
        
        local index = self.listframe:GetSelected()
        if not index or index > #errorlist then
            index = 1
            self.listframe:SetSelected(1)
        end
        self.listframe:Update()
        self.editbox:SetText(self:GetMessage(index))
    else
        MinimapButton:Hide()
        self:Hide()
    end
end

function ErrorFrame:Init()
    local WIDTH, HEIGHT, LISTHEIGHT, ITEMHEIGHT = 600, 500, 140, 20
    
    local listframe = Widget('Listbox'):New(ErrorFrame)
    listframe:SetPoint('TOPLEFT', 20, -50)
    listframe:SetPoint('TOPRIGHT', -20, -50)
    listframe:SetHeight(LISTHEIGHT)
    listframe:SetExtraButton(nil, true)
    listframe:SetLabelText(L['Error List'])
    listframe:SetList(errorlist)
    listframe:EnableSelect(true)
    listframe:SetItemHeight(ITEMHEIGHT)
    listframe.OnSelectChanged = function(self)
        ErrorFrame:Update()
    end
    listframe.RemoveClick = listframe.OnSelectChanged
    listframe.FormatButton = function(self, button)
        button:SetNormalFontObject('ChatFontNormal')
    end
    
    listframe:SetExtraNote(nil, nil, L['Delete this error?'])
    self.listframe = listframe
    
    local editbox = Widget('ScrollEditbox'):New(self)
    editbox:SetPoint('BOTTOMRIGHT', -20, 40)
    editbox:SetPoint('TOPLEFT', listframe, 'BOTTOMLEFT', 0, -20)
    editbox:SetLabelText(L['Error Message'])
    editbox:SetReadOnly(true)
    editbox:Show()
    
    local exit = Widget('Button'):New(ErrorFrame)
    exit:SetText(CLOSE)
    exit:SetPoint('BOTTOMRIGHT', -20, 15)
    exit:SetScript('OnClick', function() self:Hide() end)
    
    local button = Widget('Button'):New(ErrorFrame)
    button:SetText(L['Clear'])
    button:SetPoint('RIGHT', exit, 'LEFT', -5, 0)
    button:SetScript('OnClick', function() wipe(errorlist) self:Update() self:Hide() end)
    
    self:EnableEscape()
    self:SetSize(WIDTH, HEIGHT)
    self:SetPoint('CENTER')
    self:SetLabelText(L['taiduo\'s Error Frame'])
    self.editbox = editbox
    
    self:RegisterEvent('ADDON_ACTION_BLOCKED')
    self:RegisterEvent('MACRO_ACTION_BLOCKED')
    self:RegisterEvent('ADDON_ACTION_FORBIDDEN')
    self:RegisterEvent('MACRO_ACTION_FORBIDDEN')
    seterrorhandler(function(err)
        ErrorFrame:AddError(err, 4)
    end)
end

function MinimapButton:Init()
    self:Hide()
    self:SetMovable(true)
    self:RegisterForClicks('LeftButtonUp')
    self:RegisterForDrag('LeftButton')
    self:SetScript('OnClick', self.OnClick)
    self:SetScript('OnEnter', self.OnEnter)
    self:SetScript('OnLeave', self.OnLeave)
    self:SetScript('OnDragStart', self.OnDragStart)
    self:SetScript('OnDragStop', self.OnDragStop)
    self:UpdatePosition(config.angle or -255)
    self:SetFrameStrata('DIALOG')
    
    self:SetSize(32, 32)
    self:SetHighlightTexture([[Interface\Minimap\UI-Minimap-ZoomButton-Highlight]])
    
    local t = self:CreateTexture(nil, 'BACKGROUND')
    t:SetTexture([[Interface\Buttons\UI-MicroButton-MainMenu-Up]])
    t:SetTexCoord(0.09375, 0.90625, 0.46875, 0.89375)
    t:SetSize(20, 20)
    t:SetPoint('CENTER', -1, 1)
    
    local t = self:CreateTexture(nil, 'ARTWORK')
    t:SetTexture([[Interface\Minimap\MiniMap-TrackingBorder]])
    t:SetSize(52, 52)
    t:SetPoint('TOPLEFT')
    
    local t = self:CreateFontString(nil, 'OVERLAY', 'GameFontRedSmall')
    t:SetPoint('CENTER', -1, 2)
    self.text = t
end

function MinimapButton:UpdatePosition(angle)
    local MapScale = Minimap:GetEffectiveScale()
    local Scale = self:GetEffectiveScale()
    self:ClearAllPoints()
    self:SetPoint('CENTER', Minimap, 'TOPRIGHT', (sin(angle) * 80 - 70) * MapScale / Scale, (cos(angle) * 77 - 73) * MapScale / Scale)
end

function MinimapButton:GetAngle()
    local MapScale = Minimap:GetEffectiveScale()
    local CX, CY = GetCursorPosition()
    local X, Y = (Minimap:GetRight() - 70) * MapScale, (Minimap:GetTop() - 70) * MapScale
    
    return atan2(CY - Y, X - CX) - 90
end

function MinimapButton:OnUpdate()
    self:UpdatePosition(self:GetAngle())
end

function MinimapButton:OnDragStart()
    if IsShiftKeyDown()then
        self:OnLeave()
        self:StartUpdate()
        self:StartMoving()
    end
end

function MinimapButton:OnDragStop()
    self:StopMovingOrSizing()
    self:StopUpdate()
    self:Show()
    
    config.angle = self:GetAngle()
end

function MinimapButton:OnEnter()
    GameTooltip:SetOwner(self, "ANCHOR_TOPRIGHT")
    GameTooltip:AddLine(L['taiduo\'s Error Frame'])
    GameTooltip:AddLine(L['Click to view errors.'],1,1,1)
    GameTooltip:AddLine(L['Shift-drag to move button.'],1,1,1)
    GameTooltip:Show()
end

function MinimapButton:OnLeave()
    GameTooltip:Hide()
end

function MinimapButton:OnClick()
    ErrorFrame:Show()
end
